/* jshint latedef:false */
/* jshint forin:false */

// 
// Copyright (c) Microsoft.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

var xml = require('xml2js');
var util = require('util');
var _ = require('underscore');
var js2xml = require('../../util/js2xml');
var Service = require('../core/service');
var WebResource = require('../../http/webresource');

/**
 * The SQL Database Management API is a REST API for managing SQL Database
 * servers and the firewall rules associated with SQL Database servers.  (see
 * http://msdn.microsoft.com/en-us/library/windowsazure/gg715283.aspx for more
 * information)
 */
var SqlManagementClient = (function() {
  /**
   * Initializes a new instance of the SqlManagementClient class.
   *
   * @constructor
   *
   * @param {SubscriptionCloudCredentials} credentials - When you create a
   * Windows Azure subscription, it is uniquely identified by a subscription
   * ID. The subscription ID forms part of the URI for every call that you
   * make to the Service Management API.  The Windows Azure Service
   * ManagementAPI use mutual authentication of management certificates over
   * SSL to ensure that a request made to the service is secure.  No anonymous
   * requests are allowed.
   *
   * @param {String} baseUri - The URI used as the base for all SQL requests.
   */
  function SqlManagementClient(credentials, baseUri) {
    if (credentials === null || credentials === undefined) {
      throw new Error('credentials cannot be null.');
    }
    
    SqlManagementClient['super_'].call(this, credentials);
    
    this.credentials = credentials;
    this.baseUri = baseUri;
    if (this.baseUri === null || this.baseUri === undefined) {
      this.baseUri = 'https://management.core.windows.net';
    }
    this.dacs = new DacOperations(this);
    this.databases = new DatabaseOperations(this);
    this.firewallRules = new FirewallRuleOperations(this);
    this.servers = new ServerOperations(this);
  }
  
  util.inherits(SqlManagementClient, Service);
  
  
  return SqlManagementClient;
})();
exports.SqlManagementClient = SqlManagementClient;

/**
 * The SQL DAC Management API includes operations for importing and exporting
 * SQL Databases into and out of Windows Azure.
 */
var DacOperations = (function() {
  /**
   * Initializes a new instance of the DacOperations class.
   *
   * @constructor
   *
   * @param {SqlManagementClient} client - Reference to the service client.
   */
  function DacOperations(client) {
    this.client = client;
  }
  
  /**
   *
   * @param {String} serverName - The name of the server being imported to or
   * exported from
   *
   * @param {DacExportParameters} parametersOrCallback
   *
   * @param {Function} callback
   *
   * @returns {DacImportExportResponse} - Response for an DAC Import/Export
   * request.
   */
  DacOperations.prototype.export = function(serverName, parametersOrCallback, callback) {
    var parameters = parametersOrCallback;
    if (_.isFunction(parameters)) {
      callback = parameters;
      parameters = null;
    }
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (serverName === null || serverName === undefined) {
      return callback(new Error('serverName cannot be null.'));
    }
    if (parameters) {
      if (parameters.blobCredentials) {
        if (parameters.blobCredentials.storageAccessKey === null || parameters.blobCredentials.storageAccessKey === undefined) {
          return callback(new Error('parameters.blobCredentials.storageAccessKey cannot be null.'));
        }
        if (parameters.blobCredentials.uri === null || parameters.blobCredentials.uri === undefined) {
          return callback(new Error('parameters.blobCredentials.uri cannot be null.'));
        }
      }
      if (parameters.connectionInfo) {
        if (parameters.connectionInfo.databaseName === null || parameters.connectionInfo.databaseName === undefined) {
          return callback(new Error('parameters.connectionInfo.databaseName cannot be null.'));
        }
        if (parameters.connectionInfo.password === null || parameters.connectionInfo.password === undefined) {
          return callback(new Error('parameters.connectionInfo.password cannot be null.'));
        }
        if (parameters.connectionInfo.serverName === null || parameters.connectionInfo.serverName === undefined) {
          return callback(new Error('parameters.connectionInfo.serverName cannot be null.'));
        }
        if (parameters.connectionInfo.userName === null || parameters.connectionInfo.userName === undefined) {
          return callback(new Error('parameters.connectionInfo.userName cannot be null.'));
        }
      }
    }
    
    // Tracing
    
    // Construct URL
    var url2 = this.client.baseUri + '/' + this.client.credentials.subscriptionId + '/services/sqlservers/servers/' + serverName + '/DacOperations/Export';
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'POST';
    httpRequest.headers = {};
    httpRequest.url = url2;
    // Set Headers
    httpRequest.headers['Content-Type'] = 'application/xml';
    httpRequest.headers['x-ms-version'] = '2012-03-01';
    
    // Set Credentials
    
    // Serialize Request
    var requestContent = null;
    var requestDoc = {};
    
    if (parameters) {
      var exportInputElement = js2xml.createElement('ExportInput', 'http://schemas.datacontract.org/2004/07/Microsoft.SqlServer.Management.Dac.ServiceTypes');
      js2xml.addChildElement(requestDoc, exportInputElement);
      
      if (parameters.blobCredentials) {
        var blobCredentialsElement = js2xml.createElement('BlobCredentials', 'http://schemas.datacontract.org/2004/07/Microsoft.SqlServer.Management.Dac.ServiceTypes');
        js2xml.addChildElement(exportInputElement, blobCredentialsElement);
        
        var typeAttribute = js2xml.createAttribute('type', 'http://www.w3.org/2001/XMLSchema-instance');
        js2xml.setAttributeValue(typeAttribute, 'BlobStorageAccessKeyCredentials');
        js2xml.addAttribute(blobCredentialsElement, typeAttribute);
        
        var uriElement = js2xml.createElement('Uri', 'http://schemas.datacontract.org/2004/07/Microsoft.SqlServer.Management.Dac.ServiceTypes');
        js2xml.setElementValue(uriElement, parameters.blobCredentials.uri);
        js2xml.addChildElement(blobCredentialsElement, uriElement);
        
        var storageAccessKeyElement = js2xml.createElement('StorageAccessKey', 'http://schemas.datacontract.org/2004/07/Microsoft.SqlServer.Management.Dac.ServiceTypes');
        js2xml.setElementValue(storageAccessKeyElement, parameters.blobCredentials.storageAccessKey);
        js2xml.addChildElement(blobCredentialsElement, storageAccessKeyElement);
      }
      
      if (parameters.connectionInfo) {
        var connectionInfoElement = js2xml.createElement('ConnectionInfo', 'http://schemas.datacontract.org/2004/07/Microsoft.SqlServer.Management.Dac.ServiceTypes');
        js2xml.addChildElement(exportInputElement, connectionInfoElement);
        
        var databaseNameElement = js2xml.createElement('DatabaseName', 'http://schemas.datacontract.org/2004/07/Microsoft.SqlServer.Management.Dac.ServiceTypes');
        js2xml.setElementValue(databaseNameElement, parameters.connectionInfo.databaseName);
        js2xml.addChildElement(connectionInfoElement, databaseNameElement);
        
        var passwordElement = js2xml.createElement('Password', 'http://schemas.datacontract.org/2004/07/Microsoft.SqlServer.Management.Dac.ServiceTypes');
        js2xml.setElementValue(passwordElement, parameters.connectionInfo.password);
        js2xml.addChildElement(connectionInfoElement, passwordElement);
        
        var serverNameElement = js2xml.createElement('ServerName', 'http://schemas.datacontract.org/2004/07/Microsoft.SqlServer.Management.Dac.ServiceTypes');
        js2xml.setElementValue(serverNameElement, parameters.connectionInfo.serverName);
        js2xml.addChildElement(connectionInfoElement, serverNameElement);
        
        var userNameElement = js2xml.createElement('UserName', 'http://schemas.datacontract.org/2004/07/Microsoft.SqlServer.Management.Dac.ServiceTypes');
        js2xml.setElementValue(userNameElement, parameters.connectionInfo.userName);
        js2xml.addChildElement(connectionInfoElement, userNameElement);
      }
    }
    
    requestContent = js2xml.serializeDocument(requestDoc);
    httpRequest.body = requestContent;
    httpRequest.headers['Content-Length'] = Buffer.byteLength(requestContent, 'UTF8');
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        return callback(body);
      }
      
      // Create Result
      var result = {};
      result.statusCode = statusCode;
      result.requestId = response.headers['x-ms-request-id'];
      
      // Deserialize Response
      var responseContent = body;
      var options = {};
      options.trim = true;
      options.strict = false;
      return xml.parseString(responseContent, options, function (err2, responseDoc) {
        if (err2) {
          return callback(err2);
        }
        
        var guidElement = js2xml.getElement(responseDoc, responseDoc, 'GUID', 'HTTP://SCHEMAS.MICROSOFT.COM/2003/10/SERIALIZATION/');
        if (guidElement) {
          result.guid = guidElement;
        }
        
        return callback(null, result);
      });
    });
  };
  
  /**
   *
   * @param {String} serverName - The name of the server being imported to or
   * exported from
   *
   * @param {String} fullyQualifiedServerName - The fully qualified name of the
   * server being imported to or exported from
   *
   * @param {String} username - The server's username
   *
   * @param {String} password - The server's password
   *
   * @param {String} requestId - The request ID of the operation being queried
   *
   * @param {Function} callback
   *
   * @returns {DacGetStatusResponse} - The response structure for the DAC
   * GetStatus operation
   */
  DacOperations.prototype.getStatus = function(serverName, fullyQualifiedServerName, username, password, requestId, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (serverName === null || serverName === undefined) {
      return callback(new Error('serverName cannot be null.'));
    }
    if (fullyQualifiedServerName === null || fullyQualifiedServerName === undefined) {
      return callback(new Error('fullyQualifiedServerName cannot be null.'));
    }
    if (username === null || username === undefined) {
      return callback(new Error('username cannot be null.'));
    }
    if (password === null || password === undefined) {
      return callback(new Error('password cannot be null.'));
    }
    if (requestId === null || requestId === undefined) {
      return callback(new Error('requestId cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var url2 = this.client.baseUri + '/' + this.client.credentials.subscriptionId + '/services/sqlservers/servers/' + serverName + '/DacOperations/Status?servername=' + fullyQualifiedServerName + '&username=' + username + '&password=' + password + '&reqId=' + requestId;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'GET';
    httpRequest.headers = {};
    httpRequest.url = url2;
    // Set Headers
    httpRequest.headers['x-ms-version'] = '2012-03-01';
    
    // Set Credentials
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        return callback(body);
      }
      
      // Create Result
      var result = { statusInfoList: [] };
      result.statusCode = statusCode;
      result.requestId = response.headers['x-ms-request-id'];
      
      // Deserialize Response
      var responseContent = body;
      var options = {};
      options.trim = true;
      options.strict = false;
      return xml.parseString(responseContent, options, function (err2, responseDoc) {
        if (err2) {
          return callback(err2);
        }
        
        var arrayOfStatusInfoElement = js2xml.getElement(responseDoc, responseDoc, 'ARRAYOFSTATUSINFO', 'HTTP://SCHEMAS.DATACONTRACT.ORG/2004/07/MICROSOFT.SQLSERVER.MANAGEMENT.DAC.SERVICETYPES');
        if (arrayOfStatusInfoElement) {
          if (arrayOfStatusInfoElement) {
            for (var loweredIndex1 = 0; loweredIndex1 < js2xml.getElements(responseDoc, arrayOfStatusInfoElement, 'STATUSINFO', 'HTTP://SCHEMAS.DATACONTRACT.ORG/2004/07/MICROSOFT.SQLSERVER.MANAGEMENT.DAC.SERVICETYPES').length; loweredIndex1 = loweredIndex1 + 1) {
              var statusInfoElement = js2xml.getElements(responseDoc, arrayOfStatusInfoElement, 'STATUSINFO', 'HTTP://SCHEMAS.DATACONTRACT.ORG/2004/07/MICROSOFT.SQLSERVER.MANAGEMENT.DAC.SERVICETYPES')[loweredIndex1];
              var statusInfoInstance = {};
              result.statusInfoList.push(statusInfoInstance);
              
              var blobUriElement = js2xml.getElement(responseDoc, statusInfoElement, 'BLOBURI', 'HTTP://SCHEMAS.DATACONTRACT.ORG/2004/07/MICROSOFT.SQLSERVER.MANAGEMENT.DAC.SERVICETYPES');
              if (blobUriElement) {
                var blobUriInstance = blobUriElement;
                statusInfoInstance.blobUri = blobUriInstance;
              }
              
              var databaseNameElement = js2xml.getElement(responseDoc, statusInfoElement, 'DATABASENAME', 'HTTP://SCHEMAS.DATACONTRACT.ORG/2004/07/MICROSOFT.SQLSERVER.MANAGEMENT.DAC.SERVICETYPES');
              if (databaseNameElement) {
                var databaseNameInstance = databaseNameElement;
                statusInfoInstance.databaseName = databaseNameInstance;
              }
              
              var errorMessageElement = js2xml.getElement(responseDoc, statusInfoElement, 'ERRORMESSAGE', 'HTTP://SCHEMAS.DATACONTRACT.ORG/2004/07/MICROSOFT.SQLSERVER.MANAGEMENT.DAC.SERVICETYPES');
              if (errorMessageElement) {
                var isNil = false;
                var nilAttribute = js2xml.getAttribute(responseDoc, errorMessageElement, 'NIL', 'HTTP://WWW.W3.ORG/2001/XMLSCHEMA-INSTANCE');
                if (nilAttribute) {
                  isNil = nilAttribute === 'true';
                }
                if (isNil === false) {
                  var errorMessageInstance = errorMessageElement;
                  statusInfoInstance.errorMessage = errorMessageInstance;
                }
              }
              
              var lastModifiedTimeElement = js2xml.getElement(responseDoc, statusInfoElement, 'LASTMODIFIEDTIME', 'HTTP://SCHEMAS.DATACONTRACT.ORG/2004/07/MICROSOFT.SQLSERVER.MANAGEMENT.DAC.SERVICETYPES');
              if (lastModifiedTimeElement) {
                var lastModifiedTimeInstance = new Date(lastModifiedTimeElement);
                statusInfoInstance.lastModifiedTime = lastModifiedTimeInstance;
              }
              
              var queuedTimeElement = js2xml.getElement(responseDoc, statusInfoElement, 'QUEUEDTIME', 'HTTP://SCHEMAS.DATACONTRACT.ORG/2004/07/MICROSOFT.SQLSERVER.MANAGEMENT.DAC.SERVICETYPES');
              if (queuedTimeElement) {
                var queuedTimeInstance = new Date(queuedTimeElement);
                statusInfoInstance.queuedTime = queuedTimeInstance;
              }
              
              var requestIdElement = js2xml.getElement(responseDoc, statusInfoElement, 'REQUESTID', 'HTTP://SCHEMAS.DATACONTRACT.ORG/2004/07/MICROSOFT.SQLSERVER.MANAGEMENT.DAC.SERVICETYPES');
              if (requestIdElement) {
                var requestIdInstance = requestIdElement;
                statusInfoInstance.requestId = requestIdInstance;
              }
              
              var requestTypeElement = js2xml.getElement(responseDoc, statusInfoElement, 'REQUESTTYPE', 'HTTP://SCHEMAS.DATACONTRACT.ORG/2004/07/MICROSOFT.SQLSERVER.MANAGEMENT.DAC.SERVICETYPES');
              if (requestTypeElement) {
                var requestTypeInstance = requestTypeElement;
                statusInfoInstance.requestType = requestTypeInstance;
              }
              
              var serverNameElement = js2xml.getElement(responseDoc, statusInfoElement, 'SERVERNAME', 'HTTP://SCHEMAS.DATACONTRACT.ORG/2004/07/MICROSOFT.SQLSERVER.MANAGEMENT.DAC.SERVICETYPES');
              if (serverNameElement) {
                var serverNameInstance = serverNameElement;
                statusInfoInstance.serverName = serverNameInstance;
              }
              
              var statusElement = js2xml.getElement(responseDoc, statusInfoElement, 'STATUS', 'HTTP://SCHEMAS.DATACONTRACT.ORG/2004/07/MICROSOFT.SQLSERVER.MANAGEMENT.DAC.SERVICETYPES');
              if (statusElement) {
                var statusInstance = statusElement;
                statusInfoInstance.status = statusInstance;
              }
            }
          }
        }
        
        return callback(null, result);
      });
    });
  };
  
  /**
   *
   * @param {String} serverName - The name of the server being imported to or
   * exported from
   *
   * @param {DacImportParameters} parametersOrCallback
   *
   * @param {Function} callback
   *
   * @returns {DacImportExportResponse} - Response for an DAC Import/Export
   * request.
   */
  DacOperations.prototype.import = function(serverName, parametersOrCallback, callback) {
    var parameters = parametersOrCallback;
    if (_.isFunction(parameters)) {
      callback = parameters;
      parameters = null;
    }
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (serverName === null || serverName === undefined) {
      return callback(new Error('serverName cannot be null.'));
    }
    if (parameters) {
      if (parameters.blobCredentials) {
        if (parameters.blobCredentials.storageAccessKey === null || parameters.blobCredentials.storageAccessKey === undefined) {
          return callback(new Error('parameters.blobCredentials.storageAccessKey cannot be null.'));
        }
        if (parameters.blobCredentials.uri === null || parameters.blobCredentials.uri === undefined) {
          return callback(new Error('parameters.blobCredentials.uri cannot be null.'));
        }
      }
      if (parameters.connectionInfo) {
        if (parameters.connectionInfo.databaseName === null || parameters.connectionInfo.databaseName === undefined) {
          return callback(new Error('parameters.connectionInfo.databaseName cannot be null.'));
        }
        if (parameters.connectionInfo.password === null || parameters.connectionInfo.password === undefined) {
          return callback(new Error('parameters.connectionInfo.password cannot be null.'));
        }
        if (parameters.connectionInfo.serverName === null || parameters.connectionInfo.serverName === undefined) {
          return callback(new Error('parameters.connectionInfo.serverName cannot be null.'));
        }
        if (parameters.connectionInfo.userName === null || parameters.connectionInfo.userName === undefined) {
          return callback(new Error('parameters.connectionInfo.userName cannot be null.'));
        }
      }
      if (parameters.databaseSizeInGB === null || parameters.databaseSizeInGB === undefined) {
        return callback(new Error('parameters.databaseSizeInGB cannot be null.'));
      }
    }
    
    // Tracing
    
    // Construct URL
    var url2 = this.client.baseUri + '/' + this.client.credentials.subscriptionId + '/services/sqlservers/servers/' + serverName + '/DacOperations/Import';
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'POST';
    httpRequest.headers = {};
    httpRequest.url = url2;
    // Set Headers
    httpRequest.headers['Content-Type'] = 'application/xml';
    httpRequest.headers['x-ms-version'] = '2012-03-01';
    
    // Set Credentials
    
    // Serialize Request
    var requestContent = null;
    var requestDoc = {};
    
    if (parameters) {
      var importInputElement = js2xml.createElement('ImportInput', 'http://schemas.datacontract.org/2004/07/Microsoft.SqlServer.Management.Dac.ServiceTypes');
      js2xml.addChildElement(requestDoc, importInputElement);
      
      if (parameters.blobCredentials) {
        var blobCredentialsElement = js2xml.createElement('BlobCredentials', 'http://schemas.datacontract.org/2004/07/Microsoft.SqlServer.Management.Dac.ServiceTypes');
        js2xml.addChildElement(importInputElement, blobCredentialsElement);
        
        var typeAttribute = js2xml.createAttribute('type', 'http://www.w3.org/2001/XMLSchema-instance');
        js2xml.setAttributeValue(typeAttribute, 'BlobStorageAccessKeyCredentials');
        js2xml.addAttribute(blobCredentialsElement, typeAttribute);
        
        var uriElement = js2xml.createElement('Uri', 'http://schemas.datacontract.org/2004/07/Microsoft.SqlServer.Management.Dac.ServiceTypes');
        js2xml.setElementValue(uriElement, parameters.blobCredentials.uri);
        js2xml.addChildElement(blobCredentialsElement, uriElement);
        
        var storageAccessKeyElement = js2xml.createElement('StorageAccessKey', 'http://schemas.datacontract.org/2004/07/Microsoft.SqlServer.Management.Dac.ServiceTypes');
        js2xml.setElementValue(storageAccessKeyElement, parameters.blobCredentials.storageAccessKey);
        js2xml.addChildElement(blobCredentialsElement, storageAccessKeyElement);
      }
      
      if (parameters.connectionInfo) {
        var connectionInfoElement = js2xml.createElement('ConnectionInfo', 'http://schemas.datacontract.org/2004/07/Microsoft.SqlServer.Management.Dac.ServiceTypes');
        js2xml.addChildElement(importInputElement, connectionInfoElement);
        
        var databaseNameElement = js2xml.createElement('DatabaseName', 'http://schemas.datacontract.org/2004/07/Microsoft.SqlServer.Management.Dac.ServiceTypes');
        js2xml.setElementValue(databaseNameElement, parameters.connectionInfo.databaseName);
        js2xml.addChildElement(connectionInfoElement, databaseNameElement);
        
        var passwordElement = js2xml.createElement('Password', 'http://schemas.datacontract.org/2004/07/Microsoft.SqlServer.Management.Dac.ServiceTypes');
        js2xml.setElementValue(passwordElement, parameters.connectionInfo.password);
        js2xml.addChildElement(connectionInfoElement, passwordElement);
        
        var serverNameElement = js2xml.createElement('ServerName', 'http://schemas.datacontract.org/2004/07/Microsoft.SqlServer.Management.Dac.ServiceTypes');
        js2xml.setElementValue(serverNameElement, parameters.connectionInfo.serverName);
        js2xml.addChildElement(connectionInfoElement, serverNameElement);
        
        var userNameElement = js2xml.createElement('UserName', 'http://schemas.datacontract.org/2004/07/Microsoft.SqlServer.Management.Dac.ServiceTypes');
        js2xml.setElementValue(userNameElement, parameters.connectionInfo.userName);
        js2xml.addChildElement(connectionInfoElement, userNameElement);
      }
      
      var databaseSizeInGBElement = js2xml.createElement('DatabaseSizeInGB', 'http://schemas.datacontract.org/2004/07/Microsoft.SqlServer.Management.Dac.ServiceTypes');
      js2xml.setElementValue(databaseSizeInGBElement, parameters.databaseSizeInGB.toString());
      js2xml.addChildElement(importInputElement, databaseSizeInGBElement);
    }
    
    requestContent = js2xml.serializeDocument(requestDoc);
    httpRequest.body = requestContent;
    httpRequest.headers['Content-Length'] = Buffer.byteLength(requestContent, 'UTF8');
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        return callback(body);
      }
      
      // Create Result
      var result = {};
      result.statusCode = statusCode;
      result.requestId = response.headers['x-ms-request-id'];
      
      // Deserialize Response
      var responseContent = body;
      var options = {};
      options.trim = true;
      options.strict = false;
      return xml.parseString(responseContent, options, function (err2, responseDoc) {
        if (err2) {
          return callback(err2);
        }
        
        var guidElement = js2xml.getElement(responseDoc, responseDoc, 'GUID', 'HTTP://SCHEMAS.MICROSOFT.COM/2003/10/SERIALIZATION/');
        if (guidElement) {
          result.guid = guidElement;
        }
        
        return callback(null, result);
      });
    });
  };
  
  return DacOperations;
})();

/**
 * The SQL Database Management API includes operations for managing SQL
 * Databases for a subscription.
 */
var DatabaseOperations = (function() {
  /**
   * Initializes a new instance of the DatabaseOperations class.
   *
   * @constructor
   *
   * @param {SqlManagementClient} client - Reference to the service client.
   */
  function DatabaseOperations(client) {
    this.client = client;
  }
  
  /**
   * The Create Database operation creates a database in a SQL Server database
   * server.
   *
   * @param {String} serverName - The name of the SQL Server where the database
   * will be created
   *
   * @param {DatabaseCreateParameters} parameters - The parameters for the
   * create database operation
   *
   * @param {Function} callback
   *
   * @returns {DatabaseCreateResponse} - A standard service response including
   * an HTTP status code and request ID.
   */
  DatabaseOperations.prototype.create = function(serverName, parameters, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (serverName === null || serverName === undefined) {
      return callback(new Error('serverName cannot be null.'));
    }
    if (parameters === null || parameters === undefined) {
      return callback(new Error('parameters cannot be null.'));
    }
    if (parameters.collationName === null || parameters.collationName === undefined) {
      return callback(new Error('parameters.collationName cannot be null.'));
    }
    if (parameters.edition === null || parameters.edition === undefined) {
      return callback(new Error('parameters.edition cannot be null.'));
    }
    if (parameters.maximumDatabaseSizeInGB === null || parameters.maximumDatabaseSizeInGB === undefined) {
      return callback(new Error('parameters.maximumDatabaseSizeInGB cannot be null.'));
    }
    if (parameters.name === null || parameters.name === undefined) {
      return callback(new Error('parameters.name cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var url2 = this.client.baseUri + '/' + this.client.credentials.subscriptionId + '/services/sqlservers/servers/' + serverName + '/databases';
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'POST';
    httpRequest.headers = {};
    httpRequest.url = url2;
    // Set Headers
    httpRequest.headers['Content-Type'] = 'application/xml';
    httpRequest.headers['x-ms-version'] = '2012-03-01';
    
    // Set Credentials
    
    // Serialize Request
    var requestContent = null;
    var requestDoc = {};
    
    var serviceResourceElement = js2xml.createElement('ServiceResource', 'http://schemas.microsoft.com/windowsazure');
    js2xml.addChildElement(requestDoc, serviceResourceElement);
    
    var nameElement = js2xml.createElement('Name', 'http://schemas.microsoft.com/windowsazure');
    js2xml.setElementValue(nameElement, parameters.name);
    js2xml.addChildElement(serviceResourceElement, nameElement);
    
    var editionElement = js2xml.createElement('Edition', 'http://schemas.microsoft.com/windowsazure');
    js2xml.setElementValue(editionElement, parameters.edition);
    js2xml.addChildElement(serviceResourceElement, editionElement);
    
    var maxSizeGBElement = js2xml.createElement('MaxSizeGB', 'http://schemas.microsoft.com/windowsazure');
    js2xml.setElementValue(maxSizeGBElement, parameters.maximumDatabaseSizeInGB.toString());
    js2xml.addChildElement(serviceResourceElement, maxSizeGBElement);
    
    var collationNameElement = js2xml.createElement('CollationName', 'http://schemas.microsoft.com/windowsazure');
    js2xml.setElementValue(collationNameElement, parameters.collationName);
    js2xml.addChildElement(serviceResourceElement, collationNameElement);
    
    if (parameters.serviceObjectiveId) {
      var serviceObjectiveIdElement = js2xml.createElement('ServiceObjectiveId', 'http://schemas.microsoft.com/windowsazure');
      js2xml.setElementValue(serviceObjectiveIdElement, parameters.serviceObjectiveId);
      js2xml.addChildElement(serviceResourceElement, serviceObjectiveIdElement);
    }
    
    requestContent = js2xml.serializeDocument(requestDoc);
    httpRequest.body = requestContent;
    httpRequest.headers['Content-Length'] = Buffer.byteLength(requestContent, 'UTF8');
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 201) {
        return callback(body);
      }
      
      // Create Result
      var result = {};
      result.statusCode = statusCode;
      result.requestId = response.headers['x-ms-request-id'];
      
      // Deserialize Response
      var responseContent = body;
      var options = {};
      options.trim = true;
      options.strict = false;
      return xml.parseString(responseContent, options, function (err2, responseDoc) {
        if (err2) {
          return callback(err2);
        }
        
        var serviceResourceElement2 = js2xml.getElement(responseDoc, responseDoc, 'SERVICERESOURCE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
        if (serviceResourceElement2) {
          var nameElement2 = js2xml.getElement(responseDoc, serviceResourceElement2, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (nameElement2) {
            var nameInstance = nameElement2;
            result.name = nameInstance;
          }
          
          var idElement = js2xml.getElement(responseDoc, serviceResourceElement2, 'ID', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (idElement) {
            var idInstance = parseInt(idElement, 10);
            result.id = idInstance;
          }
          
          var typeElement = js2xml.getElement(responseDoc, serviceResourceElement2, 'TYPE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (typeElement) {
            var typeInstance = typeElement;
            result.type = typeInstance;
          }
          
          var stateElement = js2xml.getElement(responseDoc, serviceResourceElement2, 'STATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (stateElement) {
            var stateInstance = stateElement;
            result.state = stateInstance;
          }
          
          var editionElement2 = js2xml.getElement(responseDoc, serviceResourceElement2, 'EDITION', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (editionElement2) {
            var editionInstance = editionElement2;
            result.edition = editionInstance;
          }
          
          var maxSizeGBElement2 = js2xml.getElement(responseDoc, serviceResourceElement2, 'MAXSIZEGB', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (maxSizeGBElement2) {
            var maxSizeGBInstance = parseInt(maxSizeGBElement2, 10);
            result.maximumDatabaseSizeInGB = maxSizeGBInstance;
          }
          
          var collationNameElement2 = js2xml.getElement(responseDoc, serviceResourceElement2, 'COLLATIONNAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (collationNameElement2) {
            var collationNameInstance = collationNameElement2;
            result.collationName = collationNameInstance;
          }
          
          var creationDateElement = js2xml.getElement(responseDoc, serviceResourceElement2, 'CREATIONDATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (creationDateElement) {
            var creationDateInstance = new Date(creationDateElement);
            result.creationDate = creationDateInstance;
          }
          
          var isFederationRootElement = js2xml.getElement(responseDoc, serviceResourceElement2, 'ISFEDERATIONROOT', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (isFederationRootElement) {
            var isFederationRootInstance = isFederationRootElement === 'true';
            result.isFederationRoot = isFederationRootInstance;
          }
          
          var isSystemObjectElement = js2xml.getElement(responseDoc, serviceResourceElement2, 'ISSYSTEMOBJECT', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (isSystemObjectElement) {
            var isSystemObjectInstance = isSystemObjectElement === 'true';
            result.isSystemObject = isSystemObjectInstance;
          }
          
          var sizeMBElement = js2xml.getElement(responseDoc, serviceResourceElement2, 'SIZEMB', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (sizeMBElement) {
            var sizeMBInstance = sizeMBElement;
            result.sizeMB = sizeMBInstance;
          }
          
          var serviceObjectiveAssignmentErrorCodeElement = js2xml.getElement(responseDoc, serviceResourceElement2, 'SERVICEOBJECTIVEASSIGNMENTERRORCODE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (serviceObjectiveAssignmentErrorCodeElement) {
            var serviceObjectiveAssignmentErrorCodeInstance = serviceObjectiveAssignmentErrorCodeElement;
            result.serviceObjectiveAssignmentErrorCode = serviceObjectiveAssignmentErrorCodeInstance;
          }
          
          var serviceObjectiveAssignmentErrorDescriptionElement = js2xml.getElement(responseDoc, serviceResourceElement2, 'SERVICEOBJECTIVEASSIGNMENTERRORDESCRIPTION', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (serviceObjectiveAssignmentErrorDescriptionElement) {
            var serviceObjectiveAssignmentErrorDescriptionInstance = serviceObjectiveAssignmentErrorDescriptionElement;
            result.serviceObjectiveAssignmentErrorDescription = serviceObjectiveAssignmentErrorDescriptionInstance;
          }
          
          var serviceObjectiveAssignmentStateElement = js2xml.getElement(responseDoc, serviceResourceElement2, 'SERVICEOBJECTIVEASSIGNMENTSTATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (serviceObjectiveAssignmentStateElement) {
            var serviceObjectiveAssignmentStateInstance = serviceObjectiveAssignmentStateElement;
            result.serviceObjectiveAssignmentState = serviceObjectiveAssignmentStateInstance;
          }
          
          var serviceObjectiveAssignmentStateDescriptionElement = js2xml.getElement(responseDoc, serviceResourceElement2, 'SERVICEOBJECTIVEASSIGNMENTSTATEDESCRIPTION', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (serviceObjectiveAssignmentStateDescriptionElement) {
            var serviceObjectiveAssignmentStateDescriptionInstance = serviceObjectiveAssignmentStateDescriptionElement;
            result.serviceObjectiveAssignmentStateDescription = serviceObjectiveAssignmentStateDescriptionInstance;
          }
          
          var serviceObjectiveAssignmentSuccessDateElement = js2xml.getElement(responseDoc, serviceResourceElement2, 'SERVICEOBJECTIVEASSIGNMENTSUCCESSDATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (serviceObjectiveAssignmentSuccessDateElement) {
            var serviceObjectiveAssignmentSuccessDateInstance = serviceObjectiveAssignmentSuccessDateElement;
            result.serviceObjectiveAssignmentSuccessDate = serviceObjectiveAssignmentSuccessDateInstance;
          }
          
          var serviceObjectiveIdElement2 = js2xml.getElement(responseDoc, serviceResourceElement2, 'SERVICEOBJECTIVEID', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (serviceObjectiveIdElement2) {
            var serviceObjectiveIdInstance = serviceObjectiveIdElement2;
            result.serviceObjectiveId = serviceObjectiveIdInstance;
          }
        }
        
        return callback(null, result);
      });
    });
  };
  
  /**
   * The Drop Server operation drops a SQL Database server from a subscription.
   * (see http://msdn.microsoft.com/en-us/library/windowsazure/gg715285.aspx
   * for more information)
   *
   * @param {String} serverName - The name of the server on which the database
   * is found.
   *
   * @param {String} databaseName - The name of the database to be deleted.
   *
   * @param {Function} callback
   *
   * @returns {OperationResponse} - A standard service response including an
   * HTTP status code and request ID.
   */
  DatabaseOperations.prototype.delete = function(serverName, databaseName, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (serverName === null || serverName === undefined) {
      return callback(new Error('serverName cannot be null.'));
    }
    if (databaseName === null || databaseName === undefined) {
      return callback(new Error('databaseName cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var url2 = this.client.baseUri + '/' + this.client.credentials.subscriptionId + '/services/sqlservers/servers/' + serverName + '/databases/' + databaseName;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'DELETE';
    httpRequest.headers = {};
    httpRequest.url = url2;
    // Set Headers
    httpRequest.headers['x-ms-version'] = '2012-03-01';
    
    // Set Credentials
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        return callback(body);
      }
      
      // Create Result
      var result = {};
      result.statusCode = statusCode;
      result.requestId = response.headers['x-ms-request-id'];
      
      return callback(null, result);
    });
  };
  
  /**
   * The Get Database operation retrieves information about a SQL Server
   * database.
   *
   * @param {String} serverName - The name of the SQL Server on which the
   * database is housed.
   *
   * @param {String} databaseName - The name of the SQL Server database to be
   * obtained.
   *
   * @param {Function} callback
   *
   * @returns {DatabaseGetResponse} - A standard service response including an
   * HTTP status code and request ID.
   */
  DatabaseOperations.prototype.get = function(serverName, databaseName, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (serverName === null || serverName === undefined) {
      return callback(new Error('serverName cannot be null.'));
    }
    if (databaseName === null || databaseName === undefined) {
      return callback(new Error('databaseName cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var url2 = this.client.baseUri + '/' + this.client.credentials.subscriptionId + '/services/sqlservers/servers/' + serverName + '/databases/' + databaseName;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'GET';
    httpRequest.headers = {};
    httpRequest.url = url2;
    // Set Headers
    httpRequest.headers['x-ms-version'] = '2012-03-01';
    
    // Set Credentials
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        return callback(body);
      }
      
      // Create Result
      var result = {};
      result.statusCode = statusCode;
      result.requestId = response.headers['x-ms-request-id'];
      
      // Deserialize Response
      var responseContent = body;
      var options = {};
      options.trim = true;
      options.strict = false;
      return xml.parseString(responseContent, options, function (err2, responseDoc) {
        if (err2) {
          return callback(err2);
        }
        
        var serviceResourceElement = js2xml.getElement(responseDoc, responseDoc, 'SERVICERESOURCE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
        if (serviceResourceElement) {
          var nameElement = js2xml.getElement(responseDoc, serviceResourceElement, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (nameElement) {
            var nameInstance = nameElement;
            result.name = nameInstance;
          }
          
          var idElement = js2xml.getElement(responseDoc, serviceResourceElement, 'ID', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (idElement) {
            var idInstance = parseInt(idElement, 10);
            result.id = idInstance;
          }
          
          var typeElement = js2xml.getElement(responseDoc, serviceResourceElement, 'TYPE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (typeElement) {
            var typeInstance = typeElement;
            result.type = typeInstance;
          }
          
          var stateElement = js2xml.getElement(responseDoc, serviceResourceElement, 'STATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (stateElement) {
            var stateInstance = stateElement;
            result.state = stateInstance;
          }
          
          var editionElement = js2xml.getElement(responseDoc, serviceResourceElement, 'EDITION', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (editionElement) {
            var editionInstance = editionElement;
            result.edition = editionInstance;
          }
          
          var maxSizeGBElement = js2xml.getElement(responseDoc, serviceResourceElement, 'MAXSIZEGB', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (maxSizeGBElement) {
            var maxSizeGBInstance = parseInt(maxSizeGBElement, 10);
            result.maximumDatabaseSizeInGB = maxSizeGBInstance;
          }
          
          var collationNameElement = js2xml.getElement(responseDoc, serviceResourceElement, 'COLLATIONNAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (collationNameElement) {
            var collationNameInstance = collationNameElement;
            result.collationName = collationNameInstance;
          }
          
          var creationDateElement = js2xml.getElement(responseDoc, serviceResourceElement, 'CREATIONDATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (creationDateElement) {
            var creationDateInstance = new Date(creationDateElement);
            result.creationDate = creationDateInstance;
          }
          
          var isFederationRootElement = js2xml.getElement(responseDoc, serviceResourceElement, 'ISFEDERATIONROOT', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (isFederationRootElement) {
            var isFederationRootInstance = isFederationRootElement === 'true';
            result.isFederationRoot = isFederationRootInstance;
          }
          
          var isSystemObjectElement = js2xml.getElement(responseDoc, serviceResourceElement, 'ISSYSTEMOBJECT', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (isSystemObjectElement) {
            var isSystemObjectInstance = isSystemObjectElement === 'true';
            result.isSystemObject = isSystemObjectInstance;
          }
          
          var sizeMBElement = js2xml.getElement(responseDoc, serviceResourceElement, 'SIZEMB', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (sizeMBElement) {
            var sizeMBInstance = sizeMBElement;
            result.sizeMB = sizeMBInstance;
          }
          
          var serviceObjectiveAssignmentErrorCodeElement = js2xml.getElement(responseDoc, serviceResourceElement, 'SERVICEOBJECTIVEASSIGNMENTERRORCODE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (serviceObjectiveAssignmentErrorCodeElement) {
            var serviceObjectiveAssignmentErrorCodeInstance = serviceObjectiveAssignmentErrorCodeElement;
            result.serviceObjectiveAssignmentErrorCode = serviceObjectiveAssignmentErrorCodeInstance;
          }
          
          var serviceObjectiveAssignmentErrorDescriptionElement = js2xml.getElement(responseDoc, serviceResourceElement, 'SERVICEOBJECTIVEASSIGNMENTERRORDESCRIPTION', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (serviceObjectiveAssignmentErrorDescriptionElement) {
            var serviceObjectiveAssignmentErrorDescriptionInstance = serviceObjectiveAssignmentErrorDescriptionElement;
            result.serviceObjectiveAssignmentErrorDescription = serviceObjectiveAssignmentErrorDescriptionInstance;
          }
          
          var serviceObjectiveAssignmentStateElement = js2xml.getElement(responseDoc, serviceResourceElement, 'SERVICEOBJECTIVEASSIGNMENTSTATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (serviceObjectiveAssignmentStateElement) {
            var serviceObjectiveAssignmentStateInstance = serviceObjectiveAssignmentStateElement;
            result.serviceObjectiveAssignmentState = serviceObjectiveAssignmentStateInstance;
          }
          
          var serviceObjectiveAssignmentStateDescriptionElement = js2xml.getElement(responseDoc, serviceResourceElement, 'SERVICEOBJECTIVEASSIGNMENTSTATEDESCRIPTION', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (serviceObjectiveAssignmentStateDescriptionElement) {
            var serviceObjectiveAssignmentStateDescriptionInstance = serviceObjectiveAssignmentStateDescriptionElement;
            result.serviceObjectiveAssignmentStateDescription = serviceObjectiveAssignmentStateDescriptionInstance;
          }
          
          var serviceObjectiveAssignmentSuccessDateElement = js2xml.getElement(responseDoc, serviceResourceElement, 'SERVICEOBJECTIVEASSIGNMENTSUCCESSDATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (serviceObjectiveAssignmentSuccessDateElement) {
            var serviceObjectiveAssignmentSuccessDateInstance = serviceObjectiveAssignmentSuccessDateElement;
            result.serviceObjectiveAssignmentSuccessDate = serviceObjectiveAssignmentSuccessDateInstance;
          }
          
          var serviceObjectiveIdElement = js2xml.getElement(responseDoc, serviceResourceElement, 'SERVICEOBJECTIVEID', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (serviceObjectiveIdElement) {
            var serviceObjectiveIdInstance = serviceObjectiveIdElement;
            result.serviceObjectiveId = serviceObjectiveIdInstance;
          }
        }
        
        return callback(null, result);
      });
    });
  };
  
  /**
   *
   * @param {String} serverName - The name of the database server to be queried.
   *
   * @param {Function} callback
   *
   * @returns {DatabaseListResponse} - Response containing the list of
   * databases for a given server.
   */
  DatabaseOperations.prototype.list = function(serverName, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (serverName === null || serverName === undefined) {
      return callback(new Error('serverName cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var url2 = this.client.baseUri + '/' + this.client.credentials.subscriptionId + '/services/sqlservers/servers/' + serverName + '/databases?contentview=generic';
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'GET';
    httpRequest.headers = {};
    httpRequest.url = url2;
    // Set Headers
    httpRequest.headers['x-ms-version'] = '2012-03-01';
    
    // Set Credentials
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        return callback(body);
      }
      
      // Create Result
      var result = { databases: [] };
      result.statusCode = statusCode;
      result.requestId = response.headers['x-ms-request-id'];
      
      // Deserialize Response
      var responseContent = body;
      var options = {};
      options.trim = true;
      options.strict = false;
      return xml.parseString(responseContent, options, function (err2, responseDoc) {
        if (err2) {
          return callback(err2);
        }
        
        var serviceResourcesSequenceElement = js2xml.getElement(responseDoc, responseDoc, 'SERVICERESOURCES', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
        if (serviceResourcesSequenceElement) {
          for (var loweredIndex1 = 0; loweredIndex1 < js2xml.getElements(responseDoc, serviceResourcesSequenceElement, 'SERVICERESOURCE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE').length; loweredIndex1 = loweredIndex1 + 1) {
            var serviceResourcesElement = js2xml.getElements(responseDoc, serviceResourcesSequenceElement, 'SERVICERESOURCE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE')[loweredIndex1];
            var serviceResourceInstance = {};
            result.databases.push(serviceResourceInstance);
            
            var nameElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (nameElement) {
              var nameInstance = nameElement;
              serviceResourceInstance.name = nameInstance;
            }
            
            var idElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'ID', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (idElement) {
              var idInstance = parseInt(idElement, 10);
              serviceResourceInstance.id = idInstance;
            }
            
            var typeElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'TYPE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (typeElement) {
              var typeInstance = typeElement;
              serviceResourceInstance.type = typeInstance;
            }
            
            var stateElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'STATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (stateElement) {
              var stateInstance = stateElement;
              serviceResourceInstance.state = stateInstance;
            }
            
            var editionElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'EDITION', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (editionElement) {
              var editionInstance = editionElement;
              serviceResourceInstance.edition = editionInstance;
            }
            
            var maxSizeGBElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'MAXSIZEGB', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (maxSizeGBElement) {
              var maxSizeGBInstance = parseInt(maxSizeGBElement, 10);
              serviceResourceInstance.maximumDatabaseSizeInGB = maxSizeGBInstance;
            }
            
            var collationNameElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'COLLATIONNAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (collationNameElement) {
              var collationNameInstance = collationNameElement;
              serviceResourceInstance.collationName = collationNameInstance;
            }
            
            var creationDateElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'CREATIONDATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (creationDateElement) {
              var creationDateInstance = new Date(creationDateElement);
              serviceResourceInstance.creationDate = creationDateInstance;
            }
            
            var isFederationRootElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'ISFEDERATIONROOT', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (isFederationRootElement) {
              var isFederationRootInstance = isFederationRootElement === 'true';
              serviceResourceInstance.isFederationRoot = isFederationRootInstance;
            }
            
            var isSystemObjectElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'ISSYSTEMOBJECT', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (isSystemObjectElement) {
              var isSystemObjectInstance = isSystemObjectElement === 'true';
              serviceResourceInstance.isSystemObject = isSystemObjectInstance;
            }
            
            var sizeMBElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'SIZEMB', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (sizeMBElement) {
              var sizeMBInstance = sizeMBElement;
              serviceResourceInstance.sizeMB = sizeMBInstance;
            }
            
            var serviceObjectiveAssignmentErrorCodeElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'SERVICEOBJECTIVEASSIGNMENTERRORCODE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (serviceObjectiveAssignmentErrorCodeElement) {
              var serviceObjectiveAssignmentErrorCodeInstance = serviceObjectiveAssignmentErrorCodeElement;
              serviceResourceInstance.serviceObjectiveAssignmentErrorCode = serviceObjectiveAssignmentErrorCodeInstance;
            }
            
            var serviceObjectiveAssignmentErrorDescriptionElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'SERVICEOBJECTIVEASSIGNMENTERRORDESCRIPTION', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (serviceObjectiveAssignmentErrorDescriptionElement) {
              var serviceObjectiveAssignmentErrorDescriptionInstance = serviceObjectiveAssignmentErrorDescriptionElement;
              serviceResourceInstance.serviceObjectiveAssignmentErrorDescription = serviceObjectiveAssignmentErrorDescriptionInstance;
            }
            
            var serviceObjectiveAssignmentStateElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'SERVICEOBJECTIVEASSIGNMENTSTATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (serviceObjectiveAssignmentStateElement) {
              var serviceObjectiveAssignmentStateInstance = serviceObjectiveAssignmentStateElement;
              serviceResourceInstance.serviceObjectiveAssignmentState = serviceObjectiveAssignmentStateInstance;
            }
            
            var serviceObjectiveAssignmentStateDescriptionElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'SERVICEOBJECTIVEASSIGNMENTSTATEDESCRIPTION', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (serviceObjectiveAssignmentStateDescriptionElement) {
              var serviceObjectiveAssignmentStateDescriptionInstance = serviceObjectiveAssignmentStateDescriptionElement;
              serviceResourceInstance.serviceObjectiveAssignmentStateDescription = serviceObjectiveAssignmentStateDescriptionInstance;
            }
            
            var serviceObjectiveAssignmentSuccessDateElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'SERVICEOBJECTIVEASSIGNMENTSUCCESSDATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (serviceObjectiveAssignmentSuccessDateElement) {
              var serviceObjectiveAssignmentSuccessDateInstance = serviceObjectiveAssignmentSuccessDateElement;
              serviceResourceInstance.serviceObjectiveAssignmentSuccessDate = serviceObjectiveAssignmentSuccessDateInstance;
            }
            
            var serviceObjectiveIdElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'SERVICEOBJECTIVEID', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (serviceObjectiveIdElement) {
              var serviceObjectiveIdInstance = serviceObjectiveIdElement;
              serviceResourceInstance.serviceObjectiveId = serviceObjectiveIdInstance;
            }
          }
        }
        
        return callback(null, result);
      });
    });
  };
  
  /**
   * The Get Database operation retrieves information about a SQL Server
   * database.
   *
   * @param {String} serverName - The name of the SQL Server where the database
   * is housed.
   *
   * @param {String} databaseName - The name of the SQL Server database to be
   * obtained.
   *
   * @param {DatabaseUpdateParameters} parameters - The parameters for the
   * update database operation.
   *
   * @param {Function} callback
   *
   * @returns {DatabaseUpdateResponse} - A standard service response including
   * an HTTP status code and request ID.
   */
  DatabaseOperations.prototype.update = function(serverName, databaseName, parameters, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (serverName === null || serverName === undefined) {
      return callback(new Error('serverName cannot be null.'));
    }
    if (databaseName === null || databaseName === undefined) {
      return callback(new Error('databaseName cannot be null.'));
    }
    if (parameters === null || parameters === undefined) {
      return callback(new Error('parameters cannot be null.'));
    }
    if (parameters.collationName === null || parameters.collationName === undefined) {
      return callback(new Error('parameters.collationName cannot be null.'));
    }
    if (parameters.edition === null || parameters.edition === undefined) {
      return callback(new Error('parameters.edition cannot be null.'));
    }
    if (parameters.id === null || parameters.id === undefined) {
      return callback(new Error('parameters.id cannot be null.'));
    }
    if (parameters.maximumDatabaseSizeInGB === null || parameters.maximumDatabaseSizeInGB === undefined) {
      return callback(new Error('parameters.maximumDatabaseSizeInGB cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var url2 = this.client.baseUri + '/' + this.client.credentials.subscriptionId + '/services/sqlservers/servers/' + serverName + '/databases/' + databaseName;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'PUT';
    httpRequest.headers = {};
    httpRequest.url = url2;
    // Set Headers
    httpRequest.headers['Content-Type'] = 'application/xml';
    httpRequest.headers['x-ms-version'] = '2012-03-01';
    
    // Set Credentials
    
    // Serialize Request
    var requestContent = null;
    var requestDoc = {};
    
    var serviceResourceElement = js2xml.createElement('ServiceResource', 'http://schemas.microsoft.com/windowsazure');
    js2xml.addChildElement(requestDoc, serviceResourceElement);
    
    if (parameters.name) {
      var nameElement = js2xml.createElement('Name', 'http://schemas.microsoft.com/windowsazure');
      js2xml.setElementValue(nameElement, parameters.name);
      js2xml.addChildElement(serviceResourceElement, nameElement);
    }
    
    var idElement = js2xml.createElement('Id', 'http://schemas.microsoft.com/windowsazure');
    js2xml.setElementValue(idElement, parameters.id.toString());
    js2xml.addChildElement(serviceResourceElement, idElement);
    
    var editionElement = js2xml.createElement('Edition', 'http://schemas.microsoft.com/windowsazure');
    js2xml.setElementValue(editionElement, parameters.edition);
    js2xml.addChildElement(serviceResourceElement, editionElement);
    
    var maxSizeGBElement = js2xml.createElement('MaxSizeGB', 'http://schemas.microsoft.com/windowsazure');
    js2xml.setElementValue(maxSizeGBElement, parameters.maximumDatabaseSizeInGB.toString());
    js2xml.addChildElement(serviceResourceElement, maxSizeGBElement);
    
    var collationNameElement = js2xml.createElement('CollationName', 'http://schemas.microsoft.com/windowsazure');
    js2xml.setElementValue(collationNameElement, parameters.collationName);
    js2xml.addChildElement(serviceResourceElement, collationNameElement);
    
    if (parameters.serviceObjectiveId) {
      var serviceObjectiveIdElement = js2xml.createElement('ServiceObjectiveId', 'http://schemas.microsoft.com/windowsazure');
      js2xml.setElementValue(serviceObjectiveIdElement, parameters.serviceObjectiveId);
      js2xml.addChildElement(serviceResourceElement, serviceObjectiveIdElement);
    }
    
    requestContent = js2xml.serializeDocument(requestDoc);
    httpRequest.body = requestContent;
    httpRequest.headers['Content-Length'] = Buffer.byteLength(requestContent, 'UTF8');
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        return callback(body);
      }
      
      // Create Result
      var result = {};
      result.statusCode = statusCode;
      result.requestId = response.headers['x-ms-request-id'];
      
      // Deserialize Response
      var responseContent = body;
      var options = {};
      options.trim = true;
      options.strict = false;
      return xml.parseString(responseContent, options, function (err2, responseDoc) {
        if (err2) {
          return callback(err2);
        }
        
        var serviceResourceElement2 = js2xml.getElement(responseDoc, responseDoc, 'SERVICERESOURCE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
        if (serviceResourceElement2) {
          var nameElement2 = js2xml.getElement(responseDoc, serviceResourceElement2, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (nameElement2) {
            var nameInstance = nameElement2;
            result.name = nameInstance;
          }
          
          var idElement2 = js2xml.getElement(responseDoc, serviceResourceElement2, 'ID', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (idElement2) {
            var idInstance = parseInt(idElement2, 10);
            result.id = idInstance;
          }
          
          var typeElement = js2xml.getElement(responseDoc, serviceResourceElement2, 'TYPE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (typeElement) {
            var typeInstance = typeElement;
            result.type = typeInstance;
          }
          
          var stateElement = js2xml.getElement(responseDoc, serviceResourceElement2, 'STATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (stateElement) {
            var stateInstance = stateElement;
            result.state = stateInstance;
          }
          
          var editionElement2 = js2xml.getElement(responseDoc, serviceResourceElement2, 'EDITION', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (editionElement2) {
            var editionInstance = editionElement2;
            result.edition = editionInstance;
          }
          
          var maxSizeGBElement2 = js2xml.getElement(responseDoc, serviceResourceElement2, 'MAXSIZEGB', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (maxSizeGBElement2) {
            var maxSizeGBInstance = parseInt(maxSizeGBElement2, 10);
            result.maximumDatabaseSizeInGB = maxSizeGBInstance;
          }
          
          var collationNameElement2 = js2xml.getElement(responseDoc, serviceResourceElement2, 'COLLATIONNAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (collationNameElement2) {
            var collationNameInstance = collationNameElement2;
            result.collationName = collationNameInstance;
          }
          
          var creationDateElement = js2xml.getElement(responseDoc, serviceResourceElement2, 'CREATIONDATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (creationDateElement) {
            var creationDateInstance = new Date(creationDateElement);
            result.creationDate = creationDateInstance;
          }
          
          var isFederationRootElement = js2xml.getElement(responseDoc, serviceResourceElement2, 'ISFEDERATIONROOT', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (isFederationRootElement) {
            var isFederationRootInstance = isFederationRootElement === 'true';
            result.isFederationRoot = isFederationRootInstance;
          }
          
          var isSystemObjectElement = js2xml.getElement(responseDoc, serviceResourceElement2, 'ISSYSTEMOBJECT', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (isSystemObjectElement) {
            var isSystemObjectInstance = isSystemObjectElement === 'true';
            result.isSystemObject = isSystemObjectInstance;
          }
          
          var sizeMBElement = js2xml.getElement(responseDoc, serviceResourceElement2, 'SIZEMB', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (sizeMBElement) {
            var sizeMBInstance = sizeMBElement;
            result.sizeMB = sizeMBInstance;
          }
          
          var serviceObjectiveAssignmentErrorCodeElement = js2xml.getElement(responseDoc, serviceResourceElement2, 'SERVICEOBJECTIVEASSIGNMENTERRORCODE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (serviceObjectiveAssignmentErrorCodeElement) {
            var serviceObjectiveAssignmentErrorCodeInstance = serviceObjectiveAssignmentErrorCodeElement;
            result.serviceObjectiveAssignmentErrorCode = serviceObjectiveAssignmentErrorCodeInstance;
          }
          
          var serviceObjectiveAssignmentErrorDescriptionElement = js2xml.getElement(responseDoc, serviceResourceElement2, 'SERVICEOBJECTIVEASSIGNMENTERRORDESCRIPTION', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (serviceObjectiveAssignmentErrorDescriptionElement) {
            var serviceObjectiveAssignmentErrorDescriptionInstance = serviceObjectiveAssignmentErrorDescriptionElement;
            result.serviceObjectiveAssignmentErrorDescription = serviceObjectiveAssignmentErrorDescriptionInstance;
          }
          
          var serviceObjectiveAssignmentStateElement = js2xml.getElement(responseDoc, serviceResourceElement2, 'SERVICEOBJECTIVEASSIGNMENTSTATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (serviceObjectiveAssignmentStateElement) {
            var serviceObjectiveAssignmentStateInstance = serviceObjectiveAssignmentStateElement;
            result.serviceObjectiveAssignmentState = serviceObjectiveAssignmentStateInstance;
          }
          
          var serviceObjectiveAssignmentStateDescriptionElement = js2xml.getElement(responseDoc, serviceResourceElement2, 'SERVICEOBJECTIVEASSIGNMENTSTATEDESCRIPTION', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (serviceObjectiveAssignmentStateDescriptionElement) {
            var serviceObjectiveAssignmentStateDescriptionInstance = serviceObjectiveAssignmentStateDescriptionElement;
            result.serviceObjectiveAssignmentStateDescription = serviceObjectiveAssignmentStateDescriptionInstance;
          }
          
          var serviceObjectiveAssignmentSuccessDateElement = js2xml.getElement(responseDoc, serviceResourceElement2, 'SERVICEOBJECTIVEASSIGNMENTSUCCESSDATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (serviceObjectiveAssignmentSuccessDateElement) {
            var serviceObjectiveAssignmentSuccessDateInstance = serviceObjectiveAssignmentSuccessDateElement;
            result.serviceObjectiveAssignmentSuccessDate = serviceObjectiveAssignmentSuccessDateInstance;
          }
          
          var serviceObjectiveIdElement2 = js2xml.getElement(responseDoc, serviceResourceElement2, 'SERVICEOBJECTIVEID', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (serviceObjectiveIdElement2) {
            var serviceObjectiveIdInstance = serviceObjectiveIdElement2;
            result.serviceObjectiveId = serviceObjectiveIdInstance;
          }
        }
        
        return callback(null, result);
      });
    });
  };
  
  return DatabaseOperations;
})();

/**
 * The Windows Azure SQL Database Management API includes operations for
 * managing the server-level firewall rules for SQL Database servers.You
 * cannot manage the database-level firewall rules using the Windows Azure SQL
 * Database Management API; they can only be managed by running the
 * Transact-SQL statements against the master or individual user databases.
 * (see http://msdn.microsoft.com/en-us/library/windowsazure/gg715276.aspx
 * for more information)
 */
var FirewallRuleOperations = (function() {
  /**
   * Initializes a new instance of the FirewallRuleOperations class.
   *
   * @constructor
   *
   * @param {SqlManagementClient} client - Reference to the service client.
   */
  function FirewallRuleOperations(client) {
    this.client = client;
  }
  
  /**
   * The Set Server Firewall Rule operation updates an existing server-level
   * firewall rule or adds a new server-level firewall rule for a SQL Database
   * server that belongs to a subscription.  (see
   * http://msdn.microsoft.com/en-us/library/windowsazure/gg715280.aspx for
   * more information)
   *
   * @param {String} serverName - The name of the SQL database server to which
   * this rule will be applied.
   *
   * @param {FirewallRuleCreateParameters} parameters - Parameters for the
   * Create Firewall Rule operation
   *
   * @param {Function} callback
   *
   * @returns {FirewallRuleCreateResponse} - A standard service response
   * including an HTTP status code and request ID.
   */
  FirewallRuleOperations.prototype.create = function(serverName, parameters, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (serverName === null || serverName === undefined) {
      return callback(new Error('serverName cannot be null.'));
    }
    if (parameters === null || parameters === undefined) {
      return callback(new Error('parameters cannot be null.'));
    }
    if (parameters.endIPAddress === null || parameters.endIPAddress === undefined) {
      return callback(new Error('parameters.endIPAddress cannot be null.'));
    }
    if (parameters.name === null || parameters.name === undefined) {
      return callback(new Error('parameters.name cannot be null.'));
    }
    if (parameters.startIPAddress === null || parameters.startIPAddress === undefined) {
      return callback(new Error('parameters.startIPAddress cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var url2 = this.client.baseUri + '/' + this.client.credentials.subscriptionId + '/services/sqlservers/servers/' + serverName + '/firewallrules';
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'POST';
    httpRequest.headers = {};
    httpRequest.url = url2;
    // Set Headers
    httpRequest.headers['Content-Type'] = 'application/xml';
    httpRequest.headers['x-ms-version'] = '2012-03-01';
    
    // Set Credentials
    
    // Serialize Request
    var requestContent = null;
    var requestDoc = {};
    
    var serviceResourceElement = js2xml.createElement('ServiceResource', 'http://schemas.microsoft.com/windowsazure');
    js2xml.addChildElement(requestDoc, serviceResourceElement);
    
    var nameElement = js2xml.createElement('Name', 'http://schemas.microsoft.com/windowsazure');
    js2xml.setElementValue(nameElement, parameters.name);
    js2xml.addChildElement(serviceResourceElement, nameElement);
    
    var startIPAddressElement = js2xml.createElement('StartIPAddress', 'http://schemas.microsoft.com/windowsazure');
    js2xml.setElementValue(startIPAddressElement, parameters.startIPAddress);
    js2xml.addChildElement(serviceResourceElement, startIPAddressElement);
    
    var endIPAddressElement = js2xml.createElement('EndIPAddress', 'http://schemas.microsoft.com/windowsazure');
    js2xml.setElementValue(endIPAddressElement, parameters.endIPAddress);
    js2xml.addChildElement(serviceResourceElement, endIPAddressElement);
    
    requestContent = js2xml.serializeDocument(requestDoc);
    httpRequest.body = requestContent;
    httpRequest.headers['Content-Length'] = Buffer.byteLength(requestContent, 'UTF8');
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 201) {
        return callback(body);
      }
      
      // Create Result
      var result = {};
      result.statusCode = statusCode;
      result.requestId = response.headers['x-ms-request-id'];
      
      // Deserialize Response
      var responseContent = body;
      var options = {};
      options.trim = true;
      options.strict = false;
      return xml.parseString(responseContent, options, function (err2, responseDoc) {
        if (err2) {
          return callback(err2);
        }
        
        var serviceResourceElement2 = js2xml.getElement(responseDoc, responseDoc, 'SERVICERESOURCE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
        if (serviceResourceElement2) {
          var nameElement2 = js2xml.getElement(responseDoc, serviceResourceElement2, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (nameElement2) {
            var nameInstance = nameElement2;
            result.name = nameInstance;
          }
          
          var typeElement = js2xml.getElement(responseDoc, serviceResourceElement2, 'TYPE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (typeElement) {
            var typeInstance = typeElement;
            result.type = typeInstance;
          }
          
          var stateElement = js2xml.getElement(responseDoc, serviceResourceElement2, 'STATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (stateElement) {
            var stateInstance = stateElement;
            result.state = stateInstance;
          }
          
          var startIPAddressElement2 = js2xml.getElement(responseDoc, serviceResourceElement2, 'STARTIPADDRESS', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (startIPAddressElement2) {
            var startIPAddressInstance = startIPAddressElement2;
            result.startIPAddress = startIPAddressInstance;
          }
          
          var endIPAddressElement2 = js2xml.getElement(responseDoc, serviceResourceElement2, 'ENDIPADDRESS', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (endIPAddressElement2) {
            var endIPAddressInstance = endIPAddressElement2;
            result.endIPAddress = endIPAddressInstance;
          }
        }
        
        return callback(null, result);
      });
    });
  };
  
  /**
   * The Delete Server Firewall Rule operation deletes a server-level firewall
   * rule from a SQL Database server that belongs to a subscription.  (see
   * http://msdn.microsoft.com/en-us/library/windowsazure/gg715277.aspx for
   * more information)
   *
   * @param {String} serverName - The name of the server that will be have new
   * firewall rule applied to it
   *
   * @param {String} ruleName - The name of the new firewall rule
   *
   * @param {Function} callback
   *
   * @returns {OperationResponse} - A standard service response including an
   * HTTP status code and request ID.
   */
  FirewallRuleOperations.prototype.delete = function(serverName, ruleName, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (serverName === null || serverName === undefined) {
      return callback(new Error('serverName cannot be null.'));
    }
    if (ruleName === null || ruleName === undefined) {
      return callback(new Error('ruleName cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var url2 = this.client.baseUri + '/' + this.client.credentials.subscriptionId + '/services/sqlservers/servers/' + serverName + '/firewallrules/' + ruleName;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'DELETE';
    httpRequest.headers = {};
    httpRequest.url = url2;
    // Set Headers
    httpRequest.headers['x-ms-version'] = '2012-03-01';
    
    // Set Credentials
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        return callback(body);
      }
      
      // Create Result
      var result = {};
      result.statusCode = statusCode;
      result.requestId = response.headers['x-ms-request-id'];
      
      return callback(null, result);
    });
  };
  
  /**
   * The Get Server Firewall Rules operation retrieves a list of all the
   * server-level firewall rules for a SQL Database server that belongs to a
   * subscription.  (see
   * http://msdn.microsoft.com/en-us/library/windowsazure/gg715278.aspx for
   * more information)
   *
   * @param {String} serverName - The name of the server for which the call is
   * being made
   *
   * @param {Function} callback
   *
   * @returns {FirewallRuleListResponse} - A standard service response
   * including an HTTP status code and request ID.
   */
  FirewallRuleOperations.prototype.list = function(serverName, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (serverName === null || serverName === undefined) {
      return callback(new Error('serverName cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var url2 = this.client.baseUri + '/' + this.client.credentials.subscriptionId + '/services/sqlservers/servers/' + serverName + '/firewallrules';
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'GET';
    httpRequest.headers = {};
    httpRequest.url = url2;
    // Set Headers
    httpRequest.headers['x-ms-version'] = '2012-03-01';
    
    // Set Credentials
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        return callback(body);
      }
      
      // Create Result
      var result = { firewallRules: [] };
      result.statusCode = statusCode;
      result.requestId = response.headers['x-ms-request-id'];
      
      // Deserialize Response
      var responseContent = body;
      var options = {};
      options.trim = true;
      options.strict = false;
      return xml.parseString(responseContent, options, function (err2, responseDoc) {
        if (err2) {
          return callback(err2);
        }
        
        var serviceResourcesSequenceElement = js2xml.getElement(responseDoc, responseDoc, 'SERVICERESOURCES', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
        if (serviceResourcesSequenceElement) {
          for (var loweredIndex1 = 0; loweredIndex1 < js2xml.getElements(responseDoc, serviceResourcesSequenceElement, 'SERVICERESOURCE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE').length; loweredIndex1 = loweredIndex1 + 1) {
            var serviceResourcesElement = js2xml.getElements(responseDoc, serviceResourcesSequenceElement, 'SERVICERESOURCE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE')[loweredIndex1];
            var serviceResourceInstance = {};
            result.firewallRules.push(serviceResourceInstance);
            
            var nameElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (nameElement) {
              var nameInstance = nameElement;
              serviceResourceInstance.name = nameInstance;
            }
            
            var typeElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'TYPE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (typeElement) {
              var typeInstance = typeElement;
              serviceResourceInstance.type = typeInstance;
            }
            
            var startIPAddressElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'STARTIPADDRESS', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (startIPAddressElement) {
              var startIPAddressInstance = startIPAddressElement;
              serviceResourceInstance.startIPAddress = startIPAddressInstance;
            }
            
            var endIPAddressElement = js2xml.getElement(responseDoc, serviceResourcesElement, 'ENDIPADDRESS', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
            if (endIPAddressElement) {
              var endIPAddressInstance = endIPAddressElement;
              serviceResourceInstance.endIPAddress = endIPAddressInstance;
            }
          }
        }
        
        return callback(null, result);
      });
    });
  };
  
  /**
   * The Set Server Firewall Rule operation updates an existing server-level
   * firewall rule or adds a new server-level firewall rule for a SQL Database
   * server that belongs to a subscription.  (see
   * http://msdn.microsoft.com/en-us/library/windowsazure/gg715280.aspx for
   * more information)
   *
   * @param {String} serverName - The name of the SQL database server to which
   * this rule will be applied.
   *
   * @param {String} ruleName - The name of the firewall rule to be updated.
   *
   * @param {FirewallRuleUpdateParameters} parameters - Parameters for the
   * Update Firewall Rule operation
   *
   * @param {Function} callback
   *
   * @returns {FirewallRuleUpdateResponse} - A standard service response
   * including an HTTP status code and request ID.
   */
  FirewallRuleOperations.prototype.update = function(serverName, ruleName, parameters, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (serverName === null || serverName === undefined) {
      return callback(new Error('serverName cannot be null.'));
    }
    if (ruleName === null || ruleName === undefined) {
      return callback(new Error('ruleName cannot be null.'));
    }
    if (parameters === null || parameters === undefined) {
      return callback(new Error('parameters cannot be null.'));
    }
    if (parameters.endIPAddress === null || parameters.endIPAddress === undefined) {
      return callback(new Error('parameters.endIPAddress cannot be null.'));
    }
    if (parameters.name === null || parameters.name === undefined) {
      return callback(new Error('parameters.name cannot be null.'));
    }
    if (parameters.startIPAddress === null || parameters.startIPAddress === undefined) {
      return callback(new Error('parameters.startIPAddress cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var url2 = this.client.baseUri + '/' + this.client.credentials.subscriptionId + '/services/sqlservers/servers/' + serverName + '/firewallrules/' + ruleName;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'PUT';
    httpRequest.headers = {};
    httpRequest.url = url2;
    // Set Headers
    httpRequest.headers['Content-Type'] = 'application/xml';
    httpRequest.headers['x-ms-version'] = '2012-03-01';
    
    // Set Credentials
    
    // Serialize Request
    var requestContent = null;
    var requestDoc = {};
    
    var serviceResourceElement = js2xml.createElement('ServiceResource', 'http://schemas.microsoft.com/windowsazure');
    js2xml.addChildElement(requestDoc, serviceResourceElement);
    
    var nameElement = js2xml.createElement('Name', 'http://schemas.microsoft.com/windowsazure');
    js2xml.setElementValue(nameElement, parameters.name);
    js2xml.addChildElement(serviceResourceElement, nameElement);
    
    var startIPAddressElement = js2xml.createElement('StartIPAddress', 'http://schemas.microsoft.com/windowsazure');
    js2xml.setElementValue(startIPAddressElement, parameters.startIPAddress);
    js2xml.addChildElement(serviceResourceElement, startIPAddressElement);
    
    var endIPAddressElement = js2xml.createElement('EndIPAddress', 'http://schemas.microsoft.com/windowsazure');
    js2xml.setElementValue(endIPAddressElement, parameters.endIPAddress);
    js2xml.addChildElement(serviceResourceElement, endIPAddressElement);
    
    requestContent = js2xml.serializeDocument(requestDoc);
    httpRequest.body = requestContent;
    httpRequest.headers['Content-Length'] = Buffer.byteLength(requestContent, 'UTF8');
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        return callback(body);
      }
      
      // Create Result
      var result = {};
      result.statusCode = statusCode;
      result.requestId = response.headers['x-ms-request-id'];
      
      // Deserialize Response
      var responseContent = body;
      var options = {};
      options.trim = true;
      options.strict = false;
      return xml.parseString(responseContent, options, function (err2, responseDoc) {
        if (err2) {
          return callback(err2);
        }
        
        var serviceResourceElement2 = js2xml.getElement(responseDoc, responseDoc, 'SERVICERESOURCE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
        if (serviceResourceElement2) {
          var nameElement2 = js2xml.getElement(responseDoc, serviceResourceElement2, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (nameElement2) {
            var nameInstance = nameElement2;
            result.name = nameInstance;
          }
          
          var typeElement = js2xml.getElement(responseDoc, serviceResourceElement2, 'TYPE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (typeElement) {
            var typeInstance = typeElement;
            result.type = typeInstance;
          }
          
          var stateElement = js2xml.getElement(responseDoc, serviceResourceElement2, 'STATE', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (stateElement) {
            var stateInstance = stateElement;
            result.state = stateInstance;
          }
          
          var startIPAddressElement2 = js2xml.getElement(responseDoc, serviceResourceElement2, 'STARTIPADDRESS', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (startIPAddressElement2) {
            var startIPAddressInstance = startIPAddressElement2;
            result.startIPAddress = startIPAddressInstance;
          }
          
          var endIPAddressElement2 = js2xml.getElement(responseDoc, serviceResourceElement2, 'ENDIPADDRESS', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
          if (endIPAddressElement2) {
            var endIPAddressInstance = endIPAddressElement2;
            result.endIPAddress = endIPAddressInstance;
          }
        }
        
        return callback(null, result);
      });
    });
  };
  
  return FirewallRuleOperations;
})();

/**
 * The SQL Database Management API includes operations for managing SQL
 * Database servers for a subscription.  (see
 * http://msdn.microsoft.com/en-us/library/windowsazure/gg715271.aspx for more
 * information)
 */
var ServerOperations = (function() {
  /**
   * Initializes a new instance of the ServerOperations class.
   *
   * @constructor
   *
   * @param {SqlManagementClient} client - Reference to the service client.
   */
  function ServerOperations(client) {
    this.client = client;
  }
  
  /**
   * The Set Server Administrator Password operation sets the administrative
   * password of a SQL Database server for a subscription.  (see
   * http://msdn.microsoft.com/en-us/library/windowsazure/gg715272.aspx for
   * more information)
   *
   * @param {String} serverName - The server that will have the change made to
   * the administrative user
   *
   * @param {ServerChangeAdministratorPasswordParameters} parameters -
   * Parameters for the Manage Administrator Password operation
   *
   * @param {Function} callback
   *
   * @returns {OperationResponse} - A standard service response including an
   * HTTP status code and request ID.
   */
  ServerOperations.prototype.changeAdministratorPassword = function(serverName, parameters, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (serverName === null || serverName === undefined) {
      return callback(new Error('serverName cannot be null.'));
    }
    if (parameters === null || parameters === undefined) {
      return callback(new Error('parameters cannot be null.'));
    }
    if (parameters.newPassword === null || parameters.newPassword === undefined) {
      return callback(new Error('parameters.newPassword cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var url2 = this.client.baseUri + '/' + this.client.credentials.subscriptionId + '/services/sqlservers/servers/' + serverName + '?op=ResetPassword';
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'POST';
    httpRequest.headers = {};
    httpRequest.url = url2;
    // Set Headers
    httpRequest.headers['Content-Type'] = 'application/xml';
    httpRequest.headers['x-ms-version'] = '2012-03-01';
    
    // Set Credentials
    
    // Serialize Request
    var requestContent = null;
    var requestDoc = {};
    
    var administratorLoginPasswordElement = js2xml.createElement('AdministratorLoginPassword', 'http://schemas.microsoft.com/sqlazure/2010/12/');
    js2xml.addChildElement(requestDoc, administratorLoginPasswordElement);
    
    js2xml.setElementValue(administratorLoginPasswordElement, parameters.newPassword);
    
    requestContent = js2xml.serializeDocument(requestDoc);
    httpRequest.body = requestContent;
    httpRequest.headers['Content-Length'] = Buffer.byteLength(requestContent, 'UTF8');
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        return callback(body);
      }
      
      // Create Result
      var result = {};
      result.statusCode = statusCode;
      result.requestId = response.headers['x-ms-request-id'];
      
      return callback(null, result);
    });
  };
  
  /**
   * The Create Server operation adds a new SQL Database server to a
   * subscription.  (see
   * http://msdn.microsoft.com/en-us/library/windowsazure/gg715274.aspx for
   * more information)
   *
   * @param {ServerCreateParameters} parameters - Parameters supplied to the
   * Create Server operation.
   *
   * @param {Function} callback
   *
   * @returns {ServerCreateResponse} - The response returned from the Create
   * Server operation
   */
  ServerOperations.prototype.create = function(parameters, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (parameters === null || parameters === undefined) {
      return callback(new Error('parameters cannot be null.'));
    }
    if (parameters.administratorPassword === null || parameters.administratorPassword === undefined) {
      return callback(new Error('parameters.administratorPassword cannot be null.'));
    }
    if (parameters.administratorUserName === null || parameters.administratorUserName === undefined) {
      return callback(new Error('parameters.administratorUserName cannot be null.'));
    }
    if (parameters.location === null || parameters.location === undefined) {
      return callback(new Error('parameters.location cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var url2 = this.client.baseUri + '/' + this.client.credentials.subscriptionId + '/services/sqlservers/servers';
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'POST';
    httpRequest.headers = {};
    httpRequest.url = url2;
    // Set Headers
    httpRequest.headers['Content-Type'] = 'application/xml';
    httpRequest.headers['x-ms-version'] = '2012-03-01';
    
    // Set Credentials
    
    // Serialize Request
    var requestContent = null;
    var requestDoc = {};
    
    var serverElement = js2xml.createElement('Server', 'http://schemas.microsoft.com/sqlazure/2010/12/');
    js2xml.addChildElement(requestDoc, serverElement);
    
    var administratorLoginElement = js2xml.createElement('AdministratorLogin', 'http://schemas.microsoft.com/sqlazure/2010/12/');
    js2xml.setElementValue(administratorLoginElement, parameters.administratorUserName);
    js2xml.addChildElement(serverElement, administratorLoginElement);
    
    var administratorLoginPasswordElement = js2xml.createElement('AdministratorLoginPassword', 'http://schemas.microsoft.com/sqlazure/2010/12/');
    js2xml.setElementValue(administratorLoginPasswordElement, parameters.administratorPassword);
    js2xml.addChildElement(serverElement, administratorLoginPasswordElement);
    
    var locationElement = js2xml.createElement('Location', 'http://schemas.microsoft.com/sqlazure/2010/12/');
    js2xml.setElementValue(locationElement, parameters.location);
    js2xml.addChildElement(serverElement, locationElement);
    
    requestContent = js2xml.serializeDocument(requestDoc);
    httpRequest.body = requestContent;
    httpRequest.headers['Content-Length'] = Buffer.byteLength(requestContent, 'UTF8');
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 201) {
        return callback(body);
      }
      
      // Create Result
      var result = {};
      result.statusCode = statusCode;
      result.requestId = response.headers['x-ms-request-id'];
      
      // Deserialize Response
      var responseContent = body;
      var options = {};
      options.trim = true;
      options.strict = false;
      return xml.parseString(responseContent, options, function (err2, responseDoc) {
        if (err2) {
          return callback(err2);
        }
        
        var serverNameElement = js2xml.getElement(responseDoc, responseDoc, 'SERVERNAME', 'HTTP://SCHEMAS.MICROSOFT.COM/SQLAZURE/2010/12/');
        if (serverNameElement) {
          result.serverName = serverNameElement;
        }
        
        return callback(null, result);
      });
    });
  };
  
  /**
   * The Drop Server operation drops a SQL Database server from a subscription.
   * (see http://msdn.microsoft.com/en-us/library/windowsazure/gg715285.aspx
   * for more information)
   *
   * @param {String} serverName - The name of the server to be deleted
   *
   * @param {Function} callback
   *
   * @returns {OperationResponse} - A standard service response including an
   * HTTP status code and request ID.
   */
  ServerOperations.prototype.delete = function(serverName, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (serverName === null || serverName === undefined) {
      return callback(new Error('serverName cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var url2 = this.client.baseUri + '/' + this.client.credentials.subscriptionId + '/services/sqlservers/servers/' + serverName;
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'DELETE';
    httpRequest.headers = {};
    httpRequest.url = url2;
    // Set Headers
    httpRequest.headers['x-ms-version'] = '2012-03-01';
    
    // Set Credentials
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        return callback(body);
      }
      
      // Create Result
      var result = {};
      result.statusCode = statusCode;
      result.requestId = response.headers['x-ms-request-id'];
      
      return callback(null, result);
    });
  };
  
  /**
   * The Get Servers operation enumerates SQL Database servers that are
   * provisioned for a subscription.  (see
   * http://msdn.microsoft.com/en-us/library/windowsazure/gg715269.aspx for
   * more information)
   *
   * @param {Function} callback
   *
   * @returns {ServerListResponse} - The response structure for the Server List
   * operation
   */
  ServerOperations.prototype.list = function(callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    
    // Tracing
    
    // Construct URL
    var url2 = this.client.baseUri + '/' + this.client.credentials.subscriptionId + '/services/sqlservers/servers';
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'GET';
    httpRequest.headers = {};
    httpRequest.url = url2;
    // Set Headers
    httpRequest.headers['x-ms-version'] = '2012-03-01';
    
    // Set Credentials
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        return callback(body);
      }
      
      // Create Result
      var result = { servers: [] };
      result.statusCode = statusCode;
      result.requestId = response.headers['x-ms-request-id'];
      
      // Deserialize Response
      var responseContent = body;
      var options = {};
      options.trim = true;
      options.strict = false;
      return xml.parseString(responseContent, options, function (err2, responseDoc) {
        if (err2) {
          return callback(err2);
        }
        
        var serversSequenceElement = js2xml.getElement(responseDoc, responseDoc, 'SERVERS', 'HTTP://SCHEMAS.MICROSOFT.COM/SQLAZURE/2010/12/');
        if (serversSequenceElement) {
          for (var loweredIndex1 = 0; loweredIndex1 < js2xml.getElements(responseDoc, serversSequenceElement, 'SERVER', 'HTTP://SCHEMAS.MICROSOFT.COM/SQLAZURE/2010/12/').length; loweredIndex1 = loweredIndex1 + 1) {
            var serversElement = js2xml.getElements(responseDoc, serversSequenceElement, 'SERVER', 'HTTP://SCHEMAS.MICROSOFT.COM/SQLAZURE/2010/12/')[loweredIndex1];
            var serverInstance = { features: {} };
            result.servers.push(serverInstance);
            
            var nameElement = js2xml.getElement(responseDoc, serversElement, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/SQLAZURE/2010/12/');
            if (nameElement) {
              var nameInstance = nameElement;
              serverInstance.name = nameInstance;
            }
            
            var administratorLoginElement = js2xml.getElement(responseDoc, serversElement, 'ADMINISTRATORLOGIN', 'HTTP://SCHEMAS.MICROSOFT.COM/SQLAZURE/2010/12/');
            if (administratorLoginElement) {
              var administratorLoginInstance = administratorLoginElement;
              serverInstance.administratorUserName = administratorLoginInstance;
            }
            
            var locationElement = js2xml.getElement(responseDoc, serversElement, 'LOCATION', 'HTTP://SCHEMAS.MICROSOFT.COM/SQLAZURE/2010/12/');
            if (locationElement) {
              var locationInstance = locationElement;
              serverInstance.location = locationInstance;
            }
            
            var featuresSequenceElement = js2xml.getElement(responseDoc, serversElement, 'FEATURES', 'HTTP://SCHEMAS.MICROSOFT.COM/SQLAZURE/2010/12/');
            if (featuresSequenceElement) {
              for (var loweredIndex2 = 0; loweredIndex2 < js2xml.getElements(responseDoc, featuresSequenceElement, 'FEATURE', 'HTTP://SCHEMAS.MICROSOFT.COM/SQLAZURE/2010/12/').length; loweredIndex2 = loweredIndex2 + 1) {
                var featuresElement = js2xml.getElements(responseDoc, featuresSequenceElement, 'FEATURE', 'HTTP://SCHEMAS.MICROSOFT.COM/SQLAZURE/2010/12/')[loweredIndex2];
                var featuresKey = js2xml.getElement(responseDoc, featuresElement, 'NAME', 'HTTP://SCHEMAS.MICROSOFT.COM/SQLAZURE/2010/12/');
                var featuresValue = js2xml.getElement(responseDoc, featuresElement, 'VALUE', 'HTTP://SCHEMAS.MICROSOFT.COM/SQLAZURE/2010/12/');
                serverInstance.features[featuresKey] = featuresValue;
              }
            }
          }
        }
        
        return callback(null, result);
      });
    });
  };
  
  return ServerOperations;
})();
